#script returns similarity scores
#of PWMs

import math
import sys


#give input filenames
PWM_file_1 = "no_mask_HNRNPK_7mer_ranked.dat"
PWM_file_2 = "no_mask_HNRNPK_8mer_ranked.dat"

#define thresholds
spec_threshhold = 20.0


#auxiliary method
#computes the Euclidean distance
#for an alignment of rows 
#ind1 in PWM1
#ind2 in PWM2
#if ind2 is out of range of PWM2
#assume equal distribution
def distance_row(ind1,PWM1,ind2,PWM2):
	ret = 0.0
	for ind in range(0,4):
		if 0 <= ind2 < int(len(PWM2)/4):
			ret += math.pow(PWM1[ind*int(len(PWM1)/4) + ind1] - PWM2[ind*int(len(PWM2)/4) + ind2],2)
		else:
			ret += math.pow(PWM1[ind*int(len(PWM1)/4) + ind1] - 0.25,2)
	return ret


#auxiliary method 
#computes maximum Euclidean 
#similarity score
#len(PWM1) <= len(PWM2) is assumed
#assumes "half" mismatch for non-overlapping parts of PWM
def max_sim(PWM1,PWM2):
	ret = float("inf")
	ptr = 0
	for offset in range(-int(len(PWM1)/4)+1,int(len(PWM2)/4)):
		tmp = 0.0
		for i in range(0,int(len(PWM1)/4)):
			tmp += distance_row(i,PWM1,i+offset,PWM2)
		if(ret > tmp/int(len(PWM1)/4)):
			ret = tmp/int(len(PWM1)/4)
			ptr = offset
	return ret,ptr


#auxiliary method
#returns float list with next PWM
#extracted from in_file
def get_PWM(in_file):
	while True:
		line1 = in_file.readline()	#skip first line
		line1 = in_file.readline()
		line2 = in_file.readline()
		line3 = in_file.readline()
		line4 = in_file.readline()
		line5 = in_file.readline()
		
		# if two lines are empty    
		# end of file is reached
		if not (line1):
			return None
		
		#extract words from lines
		val1 = line1.split()
		val2 = line2.split()
		val3 = line3.split()
		val4 = line4.split()
		val5 = line5.split()
		
		#skip free line after each PWM
		line1 = in_file.readline()
		
		if (int(float(val5[3])) > 0 and float(val5[0]) < spec_threshhold):
			Lw = len(val1) - 1
			PWM = [0.0 for j in range(0,4*Lw)]
			for i in range(0,Lw):
				PWM[0*Lw+i] = float(val1[i+1])
			for i in range(0,Lw):
				PWM[1*Lw+i] = float(val2[i+1])
			for i in range(0,Lw):
				PWM[2*Lw+i] = float(val3[i+1])
			for i in range(0,Lw):
				PWM[3*Lw+i] = float(val4[i+1])
			return PWM
	
	return None
	

#define class of comparisons
#of two PWMs
class PWM_Comparison():
	def __init__(self, index1, index2, distance, offset, PWM1, PWM2):
		self.index1 = index1
		self.index2 = index2
		self.distance = distance
		self.offset = offset
		self.PWM1 = PWM1
		self.PWM2 = PWM2


#-------------------------------
#START OF THE SCRIPT
#-------------------------------

f2 = open(PWM_file_2,'r')
ind2 = -1
while True:
	pPWM2 = get_PWM(f2)
	
	if pPWM2 is None:
		break
	else:
		ind2 += 1
		
		comparison_array = []
		f1 = open(PWM_file_1,'r')
		ind1 = -1
		while True:
			pPWM1 = get_PWM(f1)
			
			if pPWM1 is None:
				break
			else:
				ind1 += 1
				di,of = max_sim(pPWM1,pPWM2)
				tmp_comp = PWM_Comparison(index1 = ind1, index2 = ind2, distance = di, offset = of, PWM1 = pPWM1, PWM2 = pPWM2)
				comparison_array.append(tmp_comp)
			
		f1.close()
		min_comp = min(comparison_array, key=lambda item: item.distance)
		for i in range(0,4):
			for j  in range(0,int(len(min_comp.PWM1)/4)):
				print(min_comp.PWM1[i*int(len(min_comp.PWM1)/4)+j], end = "\t")
			print("")
		print("")
		for i in range(0,4):
			for j  in range(0,int(len(min_comp.PWM2)/4)):
				print(min_comp.PWM2[i*int(len(min_comp.PWM2)/4)+j], end = "\t")
			print("")
		print(min_comp.index1,min_comp.index2,min_comp.distance,min_comp.offset)
		print("")
	
f2.close()
